package cz.cvut.kbss.reporting.model;

import java.time.LocalDateTime;
import java.util.List;
import javax.persistence.Basic;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToOne;
import javax.persistence.OrderBy;
import lombok.Getter;
import lombok.Setter;

@Getter
@Setter
@Entity
@NamedQueries( {
    @NamedQuery(name = OccurrenceReport.FIND_ALL,
        query = "SELECT r FROM OccurrenceReport r ORDER BY r.occurrence.startTime DESC"),
    @NamedQuery(name = OccurrenceReport.FIND_ALL_BY_TAG,
        query = "SELECT r FROM OccurrenceReport r WHERE :tag MEMBER OF r.tags")
})
public class OccurrenceReport extends AbstractEntity {

    public static final String FIND_ALL = "OccurrenceReport.findAll";
    public static final String FIND_ALL_BY_TAG = "OccurrenceReport.findAllByTag";

    @JoinColumn(nullable = false)
    @OneToOne(optional = false, cascade = CascadeType.ALL, fetch = FetchType.EAGER)
    private Occurrence occurrence;

    @JoinColumn(nullable = false, name = "AUTHOR_ID")
    @ManyToOne(optional = false, fetch = FetchType.EAGER)
    private Person author;

    @Basic(optional = false)
    @Column(nullable = false)
    private LocalDateTime created;

    @JoinColumn(name = "LAST_EDITOR_ID")
    @ManyToOne(fetch = FetchType.EAGER)
    private Person lastEditor;

    @OrderBy("name ASC")
    @ManyToMany(fetch = FetchType.EAGER)
    @JoinTable(name = "OCCURRENCE_REPORT_TAGS",
        joinColumns =
        @JoinColumn(name = "OCCURRENCEREPORT_ID"),
        inverseJoinColumns =
        @JoinColumn(name = "TAG_ID"))
    private List<Tag> tags;

    private LocalDateTime lastModified;

    private String narrative;

    @Override
    public String toString() {
        return "OccurrenceReport{" +
            "occurrence=" + occurrence +
            ", author=" + author +
            "}";
    }
}
