package cz.cvut.kbss.reporting.persistence.dao;

import cz.cvut.kbss.reporting.environment.Generator;
import cz.cvut.kbss.reporting.model.OccurrenceReport;
import cz.cvut.kbss.reporting.model.Person;
import cz.cvut.kbss.reporting.model.Tag;
import cz.cvut.kbss.reporting.persistence.BaseDaoTestRunner;
import java.time.LocalDateTime;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import static org.junit.Assert.assertEquals;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

public class TagDaoTest extends BaseDaoTestRunner {

    @Autowired
    private TagDao dao;

    @Autowired
    private OccurrenceReportDao reportDao;

    @Autowired
    private PersonDao personDao;

    private Person author;

    @Before
    public void setUp() {
        this.author = Generator.generatePerson();
        personDao.persist(author);
    }

    @Test
    public void findAllTags() {
        final List<OccurrenceReport> reports = generateReports();
        final Set<String> tagNames = new HashSet<>();
        reports.forEach(r -> {
            r.setAuthor(author);
            r.setCreated(LocalDateTime.now());
            final List<Tag> tags = generateRandomTags();
            tags.forEach(t -> t.setAuthor(author));
            dao.persist(tags);
            tagNames.addAll(tags.stream()
                .map(t -> t.getName())
                .collect(Collectors.toSet()));
            r.setTags(tags);
        });
        Collections.shuffle(reports);
        reportDao.persist(reports);

        assertEquals(tagNames, dao.findAll().stream()
            .map(t -> t.getName())
            .collect(Collectors.toSet())
        );
    }

    private List<Tag> generateRandomTags() {
        int max = 3;
        return IntStream.range(0, max).mapToObj(i -> Generator.generateTag(
            Character.toString((char) ('a' + Generator.randomInt(0, 5*max))))).collect(
            Collectors.toList());
    }

    private List<OccurrenceReport> generateReports() {
        return IntStream.range(0, Generator.randomInt(5, 10)).mapToObj(i -> Generator.generateReport()).collect(
                Collectors.toList());
    }
}