package cz.cvut.kbss.reporting.service.repository;

import cz.cvut.kbss.reporting.environment.Environment;
import cz.cvut.kbss.reporting.environment.Generator;
import cz.cvut.kbss.reporting.model.OccurrenceReport;
import cz.cvut.kbss.reporting.model.Person;
import cz.cvut.kbss.reporting.model.Tag;
import cz.cvut.kbss.reporting.service.BaseServiceTestRunner;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

public class OccurrenceReportServiceTest extends BaseServiceTestRunner {

    @Autowired
    private OccurrenceReportService reportService;

    private Person author;

    @Before
    public void setUp() {
        this.author = Generator.generatePerson();
        persistPerson(author);
        Environment.setCurrentUser(author);
    }

    @Test
    public void persistSetsAuthorToCurrentlyLoggedInUserAndCreatedToCurrentTime() {
        final OccurrenceReport report = Generator.generateReport();
        reportService.persist(report);

        final OccurrenceReport result = reportService.find(report.getId());
        assertNotNull(result);
        assertEquals(author, result.getAuthor());
        assertNotNull(result.getCreated());
    }

    @Test
    public void updateSetsLastEditorToCurrentlyLoggedInUserAndLastModifiedToCurrentTime() {
        final OccurrenceReport report = Generator.generateReport();
        reportService.persist(report);
        report.setNarrative("Updated narrative.");
        reportService.update(report);

        final OccurrenceReport result = reportService.find(report.getId());
        assertEquals(author, result.getLastEditor());
        assertNotNull(result.getLastModified());
        assertEquals(report.getNarrative(), result.getNarrative());
    }

//    @Test
//    public void persistCascadesForTags() {
//        final OccurrenceReport report = Generator.generateReport();
//        List<Tag> newTags = generateTags(5);
//        report.setTags(newTags);
//        reportService.persist(report);
//
//        final OccurrenceReport result = reportService.find(report.getId());
//        result.getTags().forEach(t -> {
//            assertEquals(author, t.getAuthor());
//            assertNotNull(t.getCreated());
//        });
//        assertEquals(newTags.size(), result.getTags().size());
//    }

//    @Test
//    public void updatePersistsTags() {
//        final OccurrenceReport report = Generator.generateReport();
//        report.setTags(generateTags(5));
//        reportService.persist(report);
//        final OccurrenceReport persistedReport = reportService.find(report.getId());
//        persistedReport.getTags().remove(0);
//        persistedReport.getTags().addAll(generateTags(2));
//        reportService.update(persistedReport);
//
//        final OccurrenceReport result = reportService.find(report.getId());
//        result.getTags().forEach(t -> {
//            assertEquals(author, t.getAuthor());
//            assertNotNull(t.getCreated());
//        });
//        assertEquals(5-1+2, result.getTags().size());
//    }


    private List<Tag> generateTags(int count) {
        return IntStream.range(0, count)
            .mapToObj(i -> Generator.generateTag("tag-" + Generator.randomInt()))
            .collect(Collectors.toList());
    }
}